/**
 * An object which manages a collection of variable sized sections.
 *
 * A section list is well suited to managing row heights in virtually
 * scrolling list controls. In these controls, most rows are uniform
 * height while a handful of rows are variable sized. A pair of lists
 * can be used to efficiently manage a virtually scrolling data grid.
 *
 * A section list has guaranteed `O(log n)` worst-case performance for
 * most operations, where `n` is the number of variable sized sections.
 */
export declare class SectionList {
    /**
     * Get the total number of sections in the list.
     *
     * #### Notes
     * This operation has `O(1)` complexity.
     */
    count: number;
    /**
     * Get the total size of all sections in the list.
     *
     * #### Notes
     * This operation has `O(1)` complexity.
     */
    size: number;
    /**
     * Find the index of the section which covers the given offset.
     *
     * @param offset - The positive offset position of interest.
     *
     * @returns The index of the section which covers the given offset,
     *   or `-1` if the offset is out of range.
     *
     * #### Notes
     * This operation has `O(log n)` complexity.
     */
    indexOf(offset: number): number;
    /**
     * Find the offset position of the section at the given index.
     *
     * @param index - The index of the section of interest.
     *
     * @returns The offset position of the section at the given index,
     *   or `-1` if the index is out of range.
     *
     * #### Notes
     * This operation has `O(log n)` complexity.
     */
    offsetOf(index: number): number;
    /**
     * Find the size of the section at the given index.
     *
     * @param index - The index of the section of interest.
     *
     * @returns The size of the section at the given index, or `-1` if
     *   the index is out of range.
     *
     * #### Notes
     * This operation has `O(log n)` complexity.
     */
    sizeOf(index: number): number;
    /**
     * Insert new sections into the list.
     *
     * @param index - The index at which to insert the first section.
     *   This value is clamped to the bounds of the list.
     *
     * @param count - The number of sections to insert. This must be
     *   greater than zero.
     *
     * @param size - The size of each section. This value is clamped
     *   to a lower bound of `0`.
     *
     * @returns The index of the first inserted section, or -1 if
     *   the count is `<= 0`.
     *
     * #### Notes
     * This operation has `O(log n)` complexity.
     */
    insert(index: number, count: number, size: number): number;
    /**
     * Remove existing sections from the list.
     *
     * @param index - The index of the first section to remove.
     *
     * @param count - The number of sections to remove. This must
     *   be greater than zero.
     *
     * @returns The actual number of sections removed.
     *
     * #### Notes
     * This operation has `O(log n)` complexity.
     *
     * Any sections which are out of range will be ignored.
     */
    remove(index: number, count: number): number;
    /**
     * Resize existing sections in the list.
     *
     * @param index - The index of the first section to resize.
     *
     * @param count - The number of sections to resize. This must
     *   be greater than zero.
     *
     * @param size - The new size of each section. This value is clamped
     *   to a lower bound of `0`.
     *
     * @returns The actual number of sections resized.
     *
     * #### Notes
     * This operation has `O(log n)` complexity.
     *
     * Any sections which are out of range will be ignored.
     */
    resize(index: number, count: number, size: number): number;
    private _root;
}
